﻿using Infragistics.UltraChart.Core.Primitives;
using Infragistics.UltraChart.Resources.Appearance;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace ChartLabelCollision
{
    public partial class Form1 : Form
    {
        List<Text> textBoxes = new List<Text>();

        public Form1()
        {
            InitializeComponent();

            DataTable dt = new DataTable();
            dt.Columns.Add("Numeric", typeof(int));
            dt.Columns.Add("Value", typeof(DateTime));

            dt.Rows.Add(2, DateTime.Today);

            ChartTextAppearance colAppearance = new ChartTextAppearance();
            colAppearance.Row = -2;
            colAppearance.ItemFormatString = "<SERIES_LABEL>: <ITEM_LABEL> = (<DATA_VALUE>%)";   // Something long
            colAppearance.Visible = true;
            colAppearance.HorizontalAlign = StringAlignment.Center;
            colAppearance.VerticalAlign = StringAlignment.Far;
            ultraChart1.ColumnChart.ChartText.Add(colAppearance);

            ultraChart1.DataSource = dt;
            ultraChart1.Legend.FormatString = "<SERIES_LABEL>";
            ultraChart1.Legend.Visible = true;
        }

        private void ultraChart1_FillSceneGraph(object sender, Infragistics.UltraChart.Shared.Events.FillSceneGraphEventArgs e)
        {
            // Identify the labels using the % at the end
            IEnumerable<Text> textBoxes = from Text tt in e.SceneGraph.OfType<Text>() where tt.GetTextString().EndsWith("%)") select tt;
            List<Rectangle> placed = new List<Rectangle>();
            foreach (Text tt in textBoxes)
            {
                // Get the bounds for the label
                Rectangle textBounds = GetTextBounds(tt);
                int yOverflow = Math.Max(0, textBounds.Bottom - e.ChartCore.GridLayerBounds.Bottom);

                //textBounds.Offset(0, -yOverflow);
                textBounds.Offset(0, +yOverflow);
                
                // Get a collection of the collisions and move the rectangle until it
                // doesn't collide with any rectangles that have already been checked
                IEnumerable<Rectangle> collisions = from Rectangle rr in placed where rr.IntersectsWith(textBounds) select rr;
                while (collisions.Any()) // note: potentially infinite loop
                {
                    int minY = collisions.Min((rect) => rect.Top);
                    int yOverlap = Math.Max(0, textBounds.Bottom - minY);
                    //textBounds.Offset(0, -yOverlap);
                    textBounds.Offset(0, +yOverlap);
                    collisions = from Rectangle rr in placed where rr.IntersectsWith(textBounds) select rr;
                }
                tt.bounds = textBounds;
                placed.Add(tt.bounds);
            }
        }

        private Rectangle GetTextBounds(Text text)
        {
            Size textSize = Size.Truncate(Infragistics.UltraChart.Core.Util.Platform.GetLabelSizePixels(text.GetTextString(), text.labelStyle));            
            return new Rectangle(text.bounds.Location, textSize);
        }
    }
}
